package com.dvote.dvote

import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

import dvotemobile.Dvotemobile

class DvotePlugin: MethodCallHandler {
  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "dvote")
      channel.setMethodCallHandler(DvotePlugin())
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
      "generateMnemonic" -> {
        val size = call.arguments
        if (size is Int) {
          try {
            result.success(Dvotemobile.generateMnemonic(size.toLong()))
          } catch (e: IllegalArgumentException) {
            result.error("BAD_ARGS", e.message!!, null)
          } catch (e: Exception) {
            result.error("NATIVE_ERR", e.message!!, null)
          }
        }
        else {
          result.error("BAD_ARGS", "Size must be an integer", null)
        }
      }
      "methodReceivingArgList" -> {  // TODO: Remove the example
        val args = call.arguments
        if (args is List<*>) {
          if (args.size != 2) {
            result.error("BAD_ARGS", "Wrong arg count (count expects 1 arg): ${args.size}", null)
            return
          }

          try {
            val num = readArgument<Long>(args, 0)
            // Use 'num'
            //result.success(MyPackage.method(num))
            result.success("OK")
          } catch (e: IllegalArgumentException) {
            result.error("BAD_ARGS", e.message!!, null)
          } catch (e: Exception) {
            result.error("NATIVE_ERR", e.message!!, null)
          }
        }
        else if (args is Int) {
          try {
            // result.success(MyPackage.method(args.toLong()))
            result.success("OK")
          } catch (e: IllegalArgumentException) {
            result.error("BAD_ARGS", e.message!!, null)
          } catch (e: Exception) {
            result.error("NATIVE_ERR", e.message!!, null)
          }
        }
        else {
          result.error("BAD_ARGS", "Wrong argument types", null)
        }
      }
      else -> result.notImplemented()
    }
  }

  // Helper function to extract typed argument lists
  private inline fun <reified T> readArgument(args: List<*>, index: Int): T {
    if (index >= args.size) {
      throw IllegalArgumentException("No argument available at index $index")
    }
    
    val argument = args[index]
    if (argument is T) {
      return argument
    }
    else {
      throw IllegalArgumentException("Argument at index $index " +
              "has unexpected type: ${argument?.javaClass?.name}")
    }
  }
}
