import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:dvote/dvote.dart';

void main() => runApp(MyApp());

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';
  String _mnemonic = '(mnemonic)';

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion, mnemonic;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      platformVersion = await Dvote.platformVersion;
      mnemonic = await Dvote.generateMnemonic(192);
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
      mnemonic = "(err)";
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
      _mnemonic = mnemonic;
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('DVote plugin example'),
        ),
        body: Padding(
            padding: EdgeInsets.all(24),
            child: Column(children: [
              Spacer(),
              Text('Running on: $_platformVersion'),
              SizedBox(height: 10),
              Text('Mnemonic: $_mnemonic'),
              Spacer(),
            ])),
      ),
    );
  }
}
